import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Popover, Menu, Button } from 'antd'
import { EditOutlined, FontColorsOutlined, DeleteOutlined, ToolOutlined } from '@ant-design/icons'

import asyncIconComponent from '@/utils/asyncIconComponent'
import MKEmitter from '@/utils/events.js'
import getWrapForm from './options'
import './index.scss'

const NormalForm = asyncIconComponent(() => import('@/components/normalform'))
const MenuComponent = asyncIconComponent(() => import('./menusetting'))
const LinkComponent = asyncIconComponent(() => import('./linksetting'))

const { SubMenu } = Menu

class NormalNavbar extends Component {
  static propTpyes = {
    card: PropTypes.object,
    deletecomponent: PropTypes.func,
    updateConfig: PropTypes.func,
  }

  state = {
    card: null,
    back: false
  }

  UNSAFE_componentWillMount () {
    const { card } = this.props

    if (card.isNew) {
      let _card = {
        uuid: card.uuid,
        type: card.type,
        name: card.name,
        subtype: card.subtype,
        wrap: { name: card.name, width: card.width || 1200, height: 50 },
        logoStyle: { width: '100px' },
        style: { },
        links: [],
        menus: [],
      }

      if (card.config) {
        let config = fromJS(card.config).toJS()

        _card.wrap = config.wrap
        _card.wrap.name = card.name
        _card.style = config.style
      }

      this.setState({
        card: _card
      })
      this.props.updateConfig(_card)
    } else {
      this.setState({
        card: fromJS(card).toJS()
      })
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
  }
  
  /**
   * @description 卡片行外层信息更新（数据源，样式等）
   */
  updateComponent = (component) => {
    this.setState({
      card: component
    })

    component.name = component.wrap.name

    this.props.updateConfig(component)
  }

  getStyle = (style) => {
    let _card = {...this.state.card, style}

    this.setState({
      card: _card
    })
    
    this.props.updateConfig(_card)
  }

  getLogoStyle = (style) => {
    let _card = {...this.state.card, logoStyle: style}

    this.setState({
      card: _card
    })
    
    this.props.updateConfig(_card)
  }

  changeStyle = () => {
    const { card } = this.state

    MKEmitter.emit('changeStyle', ['font', 'background', 'shadow'], card.style, this.getStyle)
  }

  changeLogoStyle = () => {
    const { card } = this.state

    MKEmitter.emit('changeStyle', ['width', 'margin'], card.logoStyle, this.getLogoStyle)
  }

  changeMenu = (menu) => {
    if (menu.property === 'link') {
      window.open(menu.link)
      return
    }
    MKEmitter.emit('changeEditMenu', {
      MenuID: menu.property === 'linkmenu' ? menu.linkMenuId : menu.MenuID,
      copyMenuId: menu.property === 'menu' ? menu.copyMenuId : '',
      clearMenu: menu.clearMenu || 'true',
      // MenuNo: menu.MenuNo,
      MenuName: menu.name,
    })
  }

  changeLogoMenu = () => {
    const { card } = this.state
  
    if (card.wrap.property === 'linkmenu') {
      MKEmitter.emit('changeEditMenu', {
        MenuID: card.wrap.linkmenu,
        copyMenuId: '',
        // MenuNo: '',
        MenuName: ''
      })
    }
  }

  getWrapForms = () => {
    const { wrap } = this.state.card

    return getWrapForm(wrap)
  }

  updateWrap = (res) => {
    this.updateComponent({...this.state.card, wrap: res})
  }

  render() {
    const { card } = this.state

    return (
      <div className="normal-navbar-edit-box" style={card.style} id={card.uuid}>
        <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
          <div className="mk-popover-control">
            <MenuComponent config={card} updateConfig={this.updateComponent} />
            <LinkComponent config={card} updateConfig={this.updateComponent} />
            <NormalForm title="导航栏设置" width={800} update={this.updateWrap} getForms={this.getWrapForms}>
              <EditOutlined style={{color: '#1890ff'}} title="编辑"/>
            </NormalForm>
            <FontColorsOutlined className="style" title="调整样式" onClick={this.changeStyle}/>
            <DeleteOutlined className="close" title="删除组件" onClick={() => this.props.deletecomponent(card.uuid)} />
          </div>
        } trigger="hover">
          <ToolOutlined />
        </Popover>
        <div className="navbar-wrap" style={{width: card.wrap.width + 'px', height: card.wrap.height + 'px', lineHeight: card.wrap.height + 'px'}}>
          {card.wrap.logo ? <Popover overlayClassName="mk-popover-control-wrap top-menu-popover" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
            <div className="mk-popover-control">
              <FontColorsOutlined className="style" title="调整样式" onClick={this.changeLogoStyle}/>
            </div>
          } trigger="hover">
            <div className={'logo' + (card.wrap.logolink ? ' pointer' : '')} style={card.logoStyle} onDoubleClick={this.changeLogoMenu}><img src={card.wrap.logo} alt=""/></div>
          </Popover> : null}
          <div className="menu">
            <Menu mode="horizontal">
              {card.menus.map(fst => {
                if (fst.property === 'classify' && fst.sublist.length > 0) {
                  return (
                    <SubMenu title={fst.name} key={fst.MenuID} popupClassName="normal-navbar-submenu">
                      {fst.sublist.map(scd => {
                        if (scd.property === 'classify' && scd.sublist.length > 0) {
                          return (
                            <Menu.ItemGroup key={scd.MenuID} title={scd.name}>
                              {scd.sublist.map(thd => {
                                return (
                                  <Menu.Item key={thd.MenuID} >
                                    <span onClick={(e) => e.stopPropagation()} onDoubleClick={() => this.changeMenu(thd)}>{thd.name}</span>
                                  </Menu.Item>
                                )
                              })}
                            </Menu.ItemGroup>
                          )
                        } else {
                          return (
                            <Menu.Item key={scd.MenuID} onClick={(e) => e.stopPropagation()}>
                              <span onClick={(e) => e.stopPropagation()} onDoubleClick={() => this.changeMenu(scd)}>{scd.name}</span>
                            </Menu.Item>
                          )
                        }
                      })}
                    </SubMenu>
                  )
                } else {
                  return (
                    <Menu.Item key={fst.MenuID}>
                      <span onClick={(e) => e.stopPropagation()} onDoubleClick={() => this.changeMenu(fst)}>{fst.name}</span>
                    </Menu.Item>
                  )
                }
              })}
            </Menu>
          </div>
          <div className="link">
            {card.links.map(link => {
              return <Button type="link" key={link.MenuID} onDoubleClick={() => this.changeMenu(link)}>{link.name}</Button>
            })}
          </div>
        </div>
      </div>
    )
  }
}

export default NormalNavbar